import redis
import json
from pydantic import BaseModel, Field
from agenix.memory.schemas import Conversation, PairMessage, Message

class RedisMemoryArgs(BaseModel):
    conversation_ttl: int = Field(default=3600, description="Time to live for a conversation in seconds")
    host: str = Field(default="localhost", description="Redis server host")
    port: int = Field(default=6379, description="Redis server port")
    db: int = Field(default=0, description="Redis database index")
    context_pairs: int = Field(default=2, description="Max number of tokens in a context")

    class Config:
        strict = True  # Жёсткая проверка типов

class RedisMemory:
    def __init__(self, args: RedisMemoryArgs):
        self.args = args
        self.redis = redis.Redis(host=args.host, port=args.port, db=args.db, decode_responses=True)

    def store_conversation(self, key: str, conversation: Conversation) -> None:
        """Сохраняет историю диалога в Redis с ограничением на количество пар."""
        # Ограничиваем количество пар согласно context_pairs
        conversation.messages = conversation.messages[-self.args.context_pairs:]
        # Сохраняем как JSON для лучшей сериализации
        self.redis.setex(key, self.args.conversation_ttl, json.dumps(conversation.model_dump()))

    def get_conversation(self, key: str) -> Conversation | None:
        """Получает историю диалога из Redis."""
        data = self.redis.get(key)
        if not data:
            return None
        conversation_data = json.loads(data)
        return Conversation(**conversation_data)

    def get_format_conversation(self, key: str) -> str:
        """Получает историю диалога из Redis в формате Markdown."""
        conversation = self.get_conversation(key)
        if conversation is None:
            return "История диалога не найдена."

        # Берем только последние context_pairs пар сообщений
        last_pairs = conversation.messages[-self.args.context_pairs:]

        formatted_messages = []
        for pair in last_pairs:
            formatted_messages.append(f"**Human:**\n{pair.human}\n\n**Assistant:**\n{pair.assistant}")
        return "\n\n".join(formatted_messages)


    def clear_conversation(self, key: str) -> None:
        """Удаляет историю диалога."""
        self.redis.delete(key)
